#! /bin/sh
#
# /etc/X11/xcalibrate/xcalibrate.sh
#
# This script is run to calibrate the touchscreen
# Run this script with '-r' option to recalibrate the screen

CONF=/etc/X11/xcalibrate/xcalibrate.conf
DRIVER26=/sys/bus/platform/drivers/ads784x/ads784x
DISPLAY=${DISPLAY:-\:0}
export DISPLAY

# Kernel environment variable that can be used to turn off calibraration.
if [ "$NOCALIBRATE" = 1 ]; then
	exit 0
fi

if [ "x$1" != "x-q" ]; then
	for N in 0 1 2 3 4 5 6 7; do
		# look for tsdev interface (Compaq ts protocol)
		if [ -d /sys/class/input/input${N}/ts0 ]; then
			unset TSLIB_TSDEVICE
			unset TSLIB_ENABLE
			# comment out the break to try tslib anyway
			break
		fi
		if [ -e /sys/class/input/input${N}/capabilities/abs ]; then
			# look for device with absolute x, y & pressure input
			ABS=`cat /sys/class/input/input${N}/capabilities/abs`
			if [ "$ABS" = "1000003" ]; then
				TSLIB_TSDEVICE=/dev/input/event$N
				export TSLIB_TSDEVICE
				TSLIB_ENABLE=1
				export TSLIB_ENABLE
				break
			fi
		fi
	done

	if [ "$TSLIB_ENABLE" ]; then
		if [ ! -f /etc/pointercal ] || [ "$1" = '-r' ]; then
			ts_calibrate > /dev/null
		fi
		exit 0
	fi
fi

# If no touchscreen driver - don't calibrate the screen
#if grep -q '11.*ts' /proc/devices || [ -L "${DRIVER26}" ]
if echo -n < /dev/ts_raw
then

# Find out if screen saver is enabled
xstat=$(xset q 2>/dev/null | grep 'timeout:')
match=$(expr match '$xstat' ' *timeout: *0.*')

xset s off
if [ ! -e ${CONF} ] || [ ! -s ${CONF} ] || [ "$1" = '-r' ]; then
	xcalibrate < /dev/null >/dev/null 2>&1
	xcalibrate -view < /dev/null > ${CONF}
else
	xcalibrate -cal $(cat ${CONF}) < /dev/null >/dev/null 2>&1
fi

if ! [ $match -eq 0 ]; then
	# SS was on before we turned it off
	xset s on
	xset s 60 5
	xset s noblank
fi

fi	# Driver detection
